@@ -16,12 +16,12 @@ def _numpy_scipy():
     # numpy versions with filters according to minimal version with a wheel
     # 1.24 limit due to https://github.com/pymor/pymor/issues/1692
     numpys = [
-        'numpy>=1.17.5,<1.24;python_version == "3.8"',
-        'numpy>=1.19.4,<1.24;python_version >= "3.9"',
+        'numpy>=1.17.5;python_version == "3.8"',
+        'numpy>=1.19.4;python_version >= "3.9"',
     ]
     scipys = [
-        'scipy>=1.3;python_version < "3.8"',
-        'scipy>=1.3.3;python_version == "3.8"',
+        'scipy>=1.5.1;python_version < "3.8"',
+        'scipy>=1.5.1;python_version == "3.8"',
         'scipy>=1.5.4;python_version >= "3.9"',
     ]
     return numpys + scipys
@@ -40,6 +40,7 @@ def setup_requires():
 #   https://github.com/jupyter-widgets/pythreejs/issues/366
 # Qt bindings selectors are a woraround for https://bugreports.qt.io/browse/QTBUG-88688
 # ipywidget pin is due to https://github.com/pymor/pymor/issues/1717
+# for jupytext and jupyter_server, see https://github.com/pymor/pymor/issues/1878
 install_requires = ['qtpy>2.0', 'packaging', 'diskcache', 'typer', 'click'] + _numpy_scipy()
 install_suggests = {
     'ipython>=5.0': 'an enhanced interactive python shell',
@@ -50,11 +51,13 @@ install_suggests = {
     'pygments': 'highlighting code',
     'pythreejs': 'threejs bindings for python notebook  visualization',
     'jupyter_client>=7.0.6': 'necessary to explicitly state here to fix 3js',
+    'jupytext>=1.14.4': 'open Markdown files in Jupyter',
+    'jupyter_server>1.3,<2.0': 'required for jupytext',
     _PYTEST: 'testing framework required to execute unit tests',
     _PYSIDE: 'solution visualization for builtin discretizations',
     'ipywidgets<8,>7': 'notebook GUI elements',
     'nbresuse': 'resource usage indicator for notebooks',
-    'torch': 'PyTorch open source machine learning framework',
+    'torch>=1.11.0': 'PyTorch open source machine learning framework',
     'jupyter_contrib_nbextensions': 'modular collection of jupyter extensions',
     'pillow': 'image library used for bitmap data functions',
     'dune-gdt>=2021.1.3; platform_system=="Linux" and platform_machine=="x86_64"': 'generic discretization toolbox',
@@ -62,23 +65,24 @@ install_suggests = {
 }
 io_requires = ['pyevtk', 'xmljson', 'meshio>=4.4', 'lxml', 'gmsh']
 install_suggests.update({p: 'optional File I/O support libraries' for p in io_requires})
-doc_requires = ['sphinx>=5.0', 'matplotlib', _PYSIDE, 'ipyparallel>=6.2.5', 'python-slugify',
+doc_requires = ['sphinx>=5.0,<5.2', 'matplotlib', _PYSIDE, 'ipyparallel>=6.2.5', 'python-slugify',
                 'ipywidgets<8,>7', 'sphinx-qt-documentation', 'bash_kernel', 'sphinx-material',
-                'sphinxcontrib-bibtex', 'sphinx-autoapi>=1.8', 'myst-nb>=0.16'] + install_requires
+                'sphinxcontrib-bibtex', 'sphinx-autoapi>=1.8,<2', 'myst-nb>=0.16'] + install_requires
 ci_requires = ['check-manifest==0.48',
                'check_reqs==0.2.0',
                'codecov==2.1.12',
                'docutils==0.18.1',
                'flake8-docstrings==1.6.0',
                'flake8-rst-docstrings==0.2.6',
-               'hypothesis[numpy,pytest]==6.48.2',
+               'hypothesis[numpy,pytest]==6.56.3',
                'pybind11==2.9.2',
-               'pypi-oldest-requirements==2021.2.0',
+               'pypi-oldest-requirements==2022.1.0',
                'pyqt5-qt5==5.15.2',
                'pyqt5==5.15.7',
                _PYTEST,
                'pytest-cov==3.0.0',
                'pytest-memprof==0.2.0',
+               'pytest-notebook==0.8.0',
                'pytest-parallel==0.1.1',
                'pytest-regressions==2.3.1',
                'pytest-xdist==2.5.0',
