@@ -254,7 +254,7 @@ class Model(CacheableObject, ParametricObject):
             If `True`, return the output estimate as a |NumPy array|,
             where each component corresponds to the respective component
             of the :attr:`output_functional`.
-            Otherwise, return the euclidian norm of all components.
+            Otherwise, return the Euclidean norm of all components.
         kwargs
             Additional keyword arguments to customize how the error estimate is
             computed or to select additional data to be returned.
@@ -278,9 +278,8 @@ class Model(CacheableObject, ParametricObject):
                 *, mu=None, input=None, **kwargs):
         """Compute the solution of the model and associated quantities.
 
-        This methods computes the output of the model it's internal state
-        and various associated quantities for given |parameter values|
-        `mu`.
+        This method computes the output of the model, its internal state,
+        and various associated quantities for given |parameter values| `mu`.
 
         .. note::
 
@@ -316,14 +315,14 @@ class Model(CacheableObject, ParametricObject):
             If `True`, return the output estimate as a |NumPy array|,
             where each component corresponds to the respective component
             of the :attr:`output_functional`.
-            Otherwise, return the euclidian norm of all components.
+            Otherwise, return the Euclidean norm of all components.
         mu
             |Parameter values| for which to compute the values.
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         kwargs
             Further keyword arguments to select further quantities that should
@@ -441,8 +440,8 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         return_error_estimate
             If `True`, also return an error estimate for the computed solution.
@@ -480,8 +479,8 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         return_error_estimate
             If `True`, also return an error estimate for the computed output.
@@ -489,7 +488,7 @@ class Model(CacheableObject, ParametricObject):
             If `True`, return the output estimate as a |NumPy array|,
             where each component corresponds to the respective component
             of the :attr:`output_functional`.
-            Otherwise, return the euclidian norm of all components.
+            Otherwise, return the Euclidean norm of all components.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
             might affect how the solution is computed.
@@ -530,8 +529,8 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
 
         Returns
@@ -556,8 +555,8 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         return_array
             if `True`, return the output gradient as a |NumPy array|.
@@ -595,8 +594,8 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
@@ -632,14 +631,14 @@ class Model(CacheableObject, ParametricObject):
         input
             The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
             a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
-            mapping time to input, or a `str` expression whith `t` as variable that
-            can be used to instatiate an |ExpressionFunction| of this type.
+            mapping time to input, or a `str` expression with `t` as variable that
+            can be used to instantiate an |ExpressionFunction| of this type.
             Can be `None` if `self.dim_input == 0`.
         return_vector
             If `True`, return the output estimate as a |NumPy array|,
             where each component corresponds to the respective component
             of the :attr:`output_functional`.
-            Otherwise, return the euclidian norm of all components.
+            Otherwise, return the Euclidean norm of all components.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
             might affect how the error estimate (or the output) is computed.
