@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from collections import namedtuple
@@ -50,23 +50,32 @@ class MPIModel:
 
 class MPIVisualizer(ImmutableObject):
 
-    def __init__(self, m_obj_id):
-        self.m_obj_id = m_obj_id
+    def __init__(self, m_obj_id, remove_model=False):
+        self.__auto_init(locals())
 
     def visualize(self, U, **kwargs):
         if isinstance(U, tuple):
-            U = tuple(u.obj_id for u in U)
+            ind = tuple(u.ind for u in U)
+            U = tuple(u.impl.obj_id for u in U)
         else:
-            U = U.obj_id
-        mpi.call(_MPIVisualizer_visualize, self.m_obj_id, U, **kwargs)
+            ind = U.ind
+            U = U.impl.obj_id
+        mpi.call(_MPIVisualizer_visualize, self.m_obj_id, U, ind, **kwargs)
+
+    def __del__(self):
+        if self.remove_model:
+            mpi.call(mpi.remove_object, self.m_obj_id)
 
 
-def _MPIVisualizer_visualize(m, U, **kwargs):
+def _MPIVisualizer_visualize(m, U, ind, **kwargs):
     m = mpi.get_object(m)
     if isinstance(U, tuple):
         U = tuple(mpi.get_object(u) for u in U)
+        U = tuple(u[i] if i is not None else u for u, i in zip(U, ind))
     else:
         U = mpi.get_object(U)
+        if ind is not None:
+            U = U[ind]
     m.visualize(U, **kwargs)
 
 
@@ -137,7 +146,9 @@ def mpi_wrap_model(local_models, mpi_spaces=('STATE',), use_with=True, with_appl
     if use_with:
         m = mpi.get_object(local_models)
         if m.visualizer:
-            wrapped_attributes['visualizer'] = MPIVisualizer(local_models)
+            wrapped_attributes['visualizer'] = MPIVisualizer(local_models, True)
+        else:
+            mpi.call(mpi.remove_object, local_models)
         m = m.with_(**wrapped_attributes)
         return m
     else:
