@@ -62,6 +62,10 @@ class Operator(ParametricObject):
         for all V, mu.
     """
 
+    # override NumPy binary operations and ufuncs
+    __array_priority__ = 100.0
+    __array_ufunc__ = None
+
     solver_options = None
 
     @property
@@ -162,7 +166,7 @@ class Operator(ParametricObject):
             op.apply_adjoint(V, mu).dot(U) == V.inner(op.apply(U, mu))
 
         Thus, when `op` is represented by a matrix `M`, `apply_adjoint` is
-        given by left-multplication of (the complex conjugate of) `M` with `V`.
+        given by left-multiplication of (the complex conjugate of) `M` with `V`.
 
         Parameters
         ----------
@@ -230,24 +234,29 @@ class Operator(ParametricObject):
 
         if self.linear:
             if solver_type is None or solver_type == 'to_matrix':
-                if solver_type is None:
-                    self.logger.warning(f'No specialized linear solver available for {self}.')
-                    self.logger.warning('Trying to solve by converting to NumPy matrix.')
-                from pymor.algorithms.rules import NoMatchingRuleError
-                try:
-                    from pymor.algorithms.to_matrix import to_matrix
-                    from pymor.operators.numpy import NumpyMatrixOperator
-                    mat = to_matrix(assembled_op)
-                    mat_op = NumpyMatrixOperator(mat)
+                mat_op = None
+                if not hasattr(self, '_mat_op'):
+                    if solver_type is None:
+                        self.logger.warning(f'No specialized linear solver available for {self}.')
+                        self.logger.warning('Trying to solve by converting to NumPy/SciPy matrix.')
+                    from pymor.algorithms.rules import NoMatchingRuleError
+                    try:
+                        from pymor.algorithms.to_matrix import to_matrix
+                        from pymor.operators.numpy import NumpyMatrixOperator
+                        mat = to_matrix(assembled_op, mu=mu)
+                        mat_op = NumpyMatrixOperator(mat)
+                        if not self.parametric:
+                            self._mat_op = mat_op
+                    except (NoMatchingRuleError, NotImplementedError):
+                        if solver_type == 'to_matrix':
+                            raise InversionError
+                        else:
+                            self.logger.warning('Failed.')
+                if mat_op is not None:
                     v = mat_op.range.from_numpy(V.to_numpy())
-                    i = None if initial_guess is None else v.source.from_numpy(initial_guess.to_numpy())
+                    i = None if initial_guess is None else mat_op.source.from_numpy(initial_guess.to_numpy())
                     u = mat_op.apply_inverse(v, initial_guess=i, least_squares=least_squares)
                     return self.source.from_numpy(u.to_numpy())
-                except (NoMatchingRuleError, NotImplementedError):
-                    if solver_type == 'to_matrix':
-                        raise InversionError
-                    else:
-                        self.logger.warning('Failed.')
             self.logger.warning('Solving with unpreconditioned iterative solver.')
             return genericsolvers.apply_inverse(assembled_op, V, initial_guess=initial_guess,
                                                 options=options, least_squares=least_squares)
