@@ -20,7 +20,8 @@ from functools import reduce
 import numpy as np
 from numpy.fft import fft, ifft, rfft, irfft
 from scipy.io import mmwrite, savemat
-from scipy.linalg import solve
+from scipy.linalg import lu_factor, lu_solve
+from scipy.linalg.lapack import get_lapack_funcs
 import scipy.sparse
 from scipy.sparse import issparse
 
@@ -324,10 +325,17 @@ class NumpyMatrixOperator(NumpyMatrixBasedOperator):
                     raise InversionError(f'{str(type(e))}: {str(e)}') from e
                 R = R.T
             else:
-                try:
-                    R = solve(self.matrix, V.to_numpy().T).T
-                except np.linalg.LinAlgError as e:
-                    raise InversionError(f'{str(type(e))}: {str(e)}') from e
+                if not hasattr(self, '_lu_factor'):
+                    try:
+                        self._lu_factor = lu_factor(self.matrix)
+                    except np.linalg.LinAlgError as e:
+                        raise InversionError(f'{str(type(e))}: {str(e)}') from e
+                    gecon = get_lapack_funcs('gecon', self._lu_factor)
+                    rcond, _ = gecon(self._lu_factor[0], np.linalg.norm(self.matrix, ord=1), norm='1')
+                    if rcond < np.finfo(np.float64).eps:
+                        self.logger.warning(f'Ill-conditioned matrix (rcond={rcond:.6g}) in apply_inverse: '
+                                            'result may not be accurate.')
+                R = lu_solve(self._lu_factor, V.to_numpy().T).T
 
             if check_finite:
                 if not np.isfinite(np.sum(R)):
@@ -388,7 +396,7 @@ class NumpyMatrixOperator(NumpyMatrixBasedOperator):
                                    solver_options=solver_options)
 
     def __getstate__(self):
-        if hasattr(self.matrix, 'factorization'):  # remove unplicklable SuperLU factorization
+        if hasattr(self.matrix, 'factorization'):  # remove unpicklable SuperLU factorization
             del self.matrix.factorization
         return self.__dict__
 
@@ -401,7 +409,7 @@ class NumpyMatrixOperator(NumpyMatrixBasedOperator):
 
 
 class NumpyHankelOperator(NumpyGenericOperator):
-    r"""Implicit representation of a Hankel operator by a |NumPy Array| of Markov parameters.
+    r"""Implicit representation of a Hankel operator by a |NumPy Array|.
 
     Let
 
@@ -411,19 +419,19 @@ class NumpyHankelOperator(NumpyGenericOperator):
             h_1 & h_2 & \dots & h_n
         \end{pmatrix},\quad h_i\in\mathbb{C}^{p\times m},\,i=1,\,\dots,\,n,\quad n,m,p\in\mathbb{N}
 
-    be a finite sequence of (matrix-valued) Markov parameters. For an odd number :math:`n=2s-1`
-    of Markov parameters, the corresponding Hankel operator can be represented by the matrix
+    be a finite (matrix-valued) sequence. If :math:`h` has an odd number :math:`n=2s-1`
+    of elements, the corresponding Hankel operator can be represented by the matrix
 
     .. math::
         H =
         \begin{bmatrix}
             h_1 & h_2 & \dots & h_s \\
             h_2 & h_3 & \dots & h_{s+1}\\
-            \vdots & \vdots && \vdots\\
+            \vdots & \vdots & \ddots & \vdots\\
             h_s & h_{s+1} & \dots & h_{2s-1}
         \end{bmatrix}\in\mathbb{C}^{ms\times ps}.
 
-    For an even number :math:`n=2s` of Markov parameters, the corresponding matrix
+    For an even number :math:`n=2s` of elements, the corresponding matrix
     representation is given by
 
     .. math::
@@ -431,29 +439,28 @@ class NumpyHankelOperator(NumpyGenericOperator):
         \begin{bmatrix}
             h_1 & h_2 & \dots & h_s & h_{s+1}\\
             h_2 & h_3 & \dots & h_{s+1} & h_{s+2}\\
-            \vdots & \vdots && \vdots & \vdots\\
+            \vdots & \vdots & \ddots & \vdots & \vdots\\
             h_s & h_{s+1} & \dots & h_{2s-1} & h_{2s}\\
             h_{s+1} & h_{s+2} & \dots & h_{2s} & 0
         \end{bmatrix}\in\mathbb{C}^{m(s+1)\times p(s+1)}.
 
-    The matrix :math:`H` as seen above is not explicitly constructed, only the sequence of Markov
-    parameters is stored. Efficient matrix-vector multiplications are realized via circulant
-    matrices with DFT in the class' `apply` method
-    (see :cite:`MSKC21` Algorithm 3.1. for details).
+    The matrix :math:`H` as seen above is not explicitly constructed, only the sequence `h` is
+    stored. Efficient matrix-vector multiplications are realized via circulant matrices with DFT in
+    the class' `apply` method (see :cite:`MSKC21` Algorithm 3.1. for details).
 
     Parameters
     ----------
-    markov_parameters
-        The |NumPy array| that contains the first :math:`n` Markov parameters that define the Hankel
-        operator. Has to be one- or three-dimensional with either::
+    h
+        The |NumPy array| that defines the Hankel operator. Has to be one- or three-dimensional with
+        either:
 
-            markov_parameters.shape = (n,)
+            h.shape = (n,)
 
-        for scalar-valued Markov parameters or::
+        for scalar-valued sequences or::
 
-            markov_parameters.shape = (n, p, m)
+            h.shape = (n, p, m)
 
-        for matrix-valued Markov parameters of dimension :math:`p\times m`.
+        for matrix-valued of sequences with elements of dimension :math:`p\times m`.
     source_id
         The id of the operator's `source` |VectorSpace|.
     range_id
@@ -462,30 +469,30 @@ class NumpyHankelOperator(NumpyGenericOperator):
         Name of the operator.
     """
 
-    def __init__(self, markov_parameters, source_id=None, range_id=None, name=None):
-        if markov_parameters.ndim == 1:
-            markov_parameters = markov_parameters.reshape(-1, 1, 1)
-        assert markov_parameters.ndim == 3
-        markov_parameters.setflags(write=False)  # make numpy arrays read-only
+    def __init__(self, h, source_id=None, range_id=None, name=None):
+        if h.ndim == 1:
+            h = h.reshape(-1, 1, 1)
+        assert h.ndim == 3
+        h.setflags(write=False)  # make numpy arrays read-only
         self.__auto_init(locals())
-        s, p, m = markov_parameters.shape
-        n = s // 2 + 1
-        self.source = NumpyVectorSpace(m * n, source_id)
-        self.range = NumpyVectorSpace(p * n, range_id)
+        n, p, m = h.shape
+        s = n // 2 + 1
+        self.source = NumpyVectorSpace(m * s, source_id)
+        self.range = NumpyVectorSpace(p * s, range_id)
         self.linear = True
-        self._circulant = self._calc_circulant()
+        self._circulant = None
 
     def apply(self, U, mu=None):
         assert U in self.source
         U = U.to_numpy().T
         k = U.shape[1]
-        s, p, m = self.markov_parameters.shape
+        s, p, m = self.h.shape
         n = s // 2 + 1
 
         FFT, iFFT = fft, ifft
-        c = self._circulant
+        c = self._calc_circulant()
         dtype = complex
-        if np.isrealobj(self.markov_parameters):
+        if np.isrealobj(self.h):
             if np.isrealobj(U):
                 FFT, iFFT = rfft, irfft
                 dtype = float
@@ -505,29 +512,14 @@ class NumpyHankelOperator(NumpyGenericOperator):
         return self.H.apply(V, mu=mu)
 
     def _calc_circulant(self):
-        FFT = rfft if np.isrealobj(self.markov_parameters) else fft
-        s, p, m = self.markov_parameters.shape
-        return FFT(
-            np.roll(
-                np.concatenate(
-                    [
-                        np.zeros([1, p, m]),
-                        self.markov_parameters,
-                        np.zeros([1 - s % 2, p, m]),
-                    ]
-                ),
-                s // 2 + 1,
-                axis=0,
-            ),
-            axis=0,
-        )
+        if self._circulant is None:
+            FFT = rfft if np.isrealobj(self.h) else fft
+            s, p, m = self.h.shape
+            self._circulant = FFT(np.roll(
+                np.concatenate([np.zeros([1, p, m]), self.h, np.zeros([1 - s % 2, p, m])]), s // 2 + 1, axis=0), axis=0)
+        return self._circulant
 
     @property
     def H(self):
-        adjoint_markov_parameters = self.markov_parameters.transpose(0, 2, 1).conj()
-        return self.with_(
-            markov_parameters=adjoint_markov_parameters,
-            source_id=self.range_id,
-            range_id=self.source_id,
-            name=self.name + '_adjoint',
-        )
+        adjoint_h = self.h.transpose(0, 2, 1).conj()
+        return self.with_(h=adjoint_h, source_id=self.range_id, range_id=self.source_id, name=self.name + '_adjoint')
