@@ -31,7 +31,7 @@ def float_cmp(x, y, rtol=1e-14, atol=1e-14):
     atol
         The absolute tolerance.
     """
-    return np.abs(x - y) <= atol + np.abs(y) * rtol
+    return np.abs(x - y) <= atol + np.abs(y * rtol)
 
 
 def float_cmp_all(x, y, rtol=None, atol=None):
@@ -81,4 +81,4 @@ def almost_less(x, y, rtol=1e-14, atol=1e-14):
     atol
         The absolute tolerance.
     """
-    return x - y <= atol + np.abs(y) * rtol
+    return x - y <= atol + np.abs(y * rtol)
