@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import sys
 import time
@@ -49,7 +49,7 @@ def histogram(
 
     print()
 
-    if hasattr(rom, 'estimate'):
+    if hasattr(rom, 'estimate_error'):
         ests = []
         for mu in mus:
             print(f'Estimating error for {mu} ... ', end='')
@@ -79,7 +79,7 @@ def histogram(
     except AttributeError:
         pass  # plt.style is only available in newer matplotlib versions
 
-    if hasattr(rom, 'estimate') and detailed_data:
+    if hasattr(rom, 'estimate_error') and detailed_data:
 
         # setup axes
         left, width = 0.1, 0.65
@@ -121,7 +121,7 @@ def histogram(
 
         plt.show()
 
-    elif hasattr(rom, 'estimate'):
+    elif hasattr(rom, 'estimate_error'):
 
         total_min = np.min(ests) * 0.9
         total_max = np.max(ests) * 1.1
@@ -169,7 +169,7 @@ def convergence(
 
     dim = rom.solution_space.dim
     if ndim:
-        dims = np.linspace(0, dim, ndim, dtype=np.int)
+        dims = np.linspace(0, dim, ndim, dtype=int)
     else:
         dims = np.arange(dim + 1)
 
@@ -241,9 +241,10 @@ def convergence(
 
 
 def _bins(start, stop, steps=100):
-    ''' numpy has a quirk in unreleased master where logspace
-    might sometimes not return a 1d array
-    '''
+    """Compute bins.
+
+    NumPy has a quirk in unreleased master where logspace might sometimes not return a 1d array.
+    """
     bins = np.logspace(np.log10(start), np.log10(stop), steps)
     if bins.shape == (steps, 1):
         bins = bins[:, 0]
